# coding:utf-8

import os
import re
import sys
import uuid
import json
import zipfile
import argparse
import platform
import requests
from pathlib import Path
from super_sweetest.client.mqtt.api_mqtt_cli import mqtt_workspace


def main():
    """
    创建命令： super_mqtt -r http://127.0.0.1 -mqtt -n 执行端名称
    api 0 / android 1 / ios 2 / web 3 / mqtt 4
    """
    if len(sys.argv) == 1:
        sys.argv.append('--help')
    parser = argparse.ArgumentParser(description='X ATP CLI Client (X 自动化测试平台命令行客户端)')
    parser.add_argument('-v', '--version', help='输出客户端版本信息', action='version', version='%(prog)s v1.1.9.7')
    parser.add_argument('-d', '--demo', help='在当前目录下创建Demo项目 `sweetest_example`', action='store_true')
    parser.add_argument('-r', '--run', dest='server_url', help='创建 X-ATP 自动测试执行端服务: super_sweetest -r http://127.0.0.1 -m -n 执行端名称', action='store')
    parser.add_argument('-m', '--mqtt', help='运行 mqtt 测试执行端 (used with parameter -r)', action='store_true', default=False)
    parser.add_argument('-n', '--name', dest='workspace_name', help='执行端工作区的标识名称 (used with parameter -r)', action='store', default='')
    parser.add_argument('-a', '--app', help='在当前目录下创建Android/Ios执行端主程序 `run_sync_devices_`', action='store_true')
    parser.add_argument('-R', '--Redis_host', dest='Redis_host', help='如果与 server_url 不同，单独配置 Redis 地址,如：-R 127.0.0.1')
    args = parser.parse_args() # 调试时传参 ['-a']
    if args.demo or args.app:
        # 解压demo项目/Android/Ios执行端主程序
        if args.demo:
            zip_name = 'sweetest_example'
            msg1 = '\n\n生成 Demo项目 `sweetest_example` 成功\n快速体验，请输入如下命令(进入示例目录，启动运行脚本):\n\ncd sweetest_example\npython start.py'
        else:
            zip_name = 'run_sync_devices_'
            msg1 = '\n\n生成 super_sweetest 创建的 Android/Ios 执行端主程序 `run_sync_devices_.py` 成功\n快速体验，请双击运行'

        sweetest_dir = Path(__file__).resolve().parents[0]
        example_dir_ = sweetest_dir / 'example' / '{}.py'.format(zip_name)
        android_ios_dir_ = sweetest_dir / 'android_ios' / '{}.py'.format(zip_name)
        if os.path.exists(example_dir_) and args.demo:
            new_example_dir = sweetest_dir / 'example' / '{}.zip'.format(zip_name)
            os.rename(example_dir_, new_example_dir)
            example_dir = sweetest_dir / 'example' / '{}.zip'.format(zip_name)
            extract(str(example_dir), Path.cwd())
        elif os.path.exists(android_ios_dir_) and args.app:
            import shutil
            shutil.copy(android_ios_dir_, './')

        print('\n文档: https://sweeter.io\n公众号：喜文测试\nsuper_sweetest 自动化 Q Q 群：630589174 (验证码：python)注意首字母小写')
        print(msg1)

    if args.server_url:
        if not re.match(r'(http|https):\/\/[\w\-_]+(\.[\w\-_]+)+([\w\-\.,@?^=%&:/~\+#]*[\w\-\@?^=%&/~\+#])?', args.server_url, re.IGNORECASE):
            print('[Cli Error]> 这不是有效的URl地址')
            return
        if args.mqtt:
            data_json = get_initialization(test_type=4, platform_url=args.server_url, workspace_name=args.workspace_name)
            mqtt_workspace(sys_name=platform.system(), data=data_json['data'], redis_host=args.Redis_host)
        else:
            print('[Cli Error]> 缺少 -api|-mqtt|-android|-ios|-web 参数')


def extract(z_file, path):
    """
    解压缩文件到指定目录
    """
    f = zipfile.ZipFile(z_file, 'r')
    for file in f.namelist():
        f.extract(file, path)


def get_initialization(test_type, platform_url, workspace_name):
    # 对输入的URL做判断，末尾没加"/" 统一处理加上"/"
    if not platform_url.endswith('/'):
        platform_url += '/'
    # ATP平台端初始化执行端的mqtt接口
    initialization_mqtt_url = platform_url + 'software/execution/initialization/'
    # 如果执行工作区名称为默认值，生成随机数
    if workspace_name == '':
        # 根据当前网卡和时间组成随机数
        workspace_name = uuid.uuid4().hex
    # mqtt、android、ios、web执行端的通用参数
    up_data = {'name': workspace_name, 'execution_type': test_type}
    # mqtt执行端的特殊参数
    if test_type == 4:
        up_data['information'] = json.dumps({'system': platform.system()})
    # 向ATP平台端发送初始化执行端请求
    requests_data = requests.post(initialization_mqtt_url, data=up_data)
    requests_json = requests_data.json()
    print('[Cli info]> ' + str(requests_json))
    # 判断平台端初始化结果
    if requests_json['code'] != 200:
        print('[Cli Error]> 执行端服务初始化失败')
        sys.exit()
    # 附带上经过上面处理的ATP平台URl
    requests_json['data']['platform_url'] = platform_url
    requests_json['data']['redis_ip'] = platform_url.replace('http://', '').replace('/', '')
    requests_json['data']['test_type'] = test_type
    return requests_json


if __name__ == '__main__':
    main()



import os
from time import sleep
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from super_sweetest.elements import e
from super_sweetest.globals import g
from super_sweetest.log import logger
from super_sweetest.config import element_wait_timeout


def locating_element(element, action=''):
    el_location = None
    try:
        el, value = e.get(element)
    except:
        logger.exception(
            'Locating the element:%s is Failure, this element is not define' % element)
        raise Exception(
            'Locating the element:%s is Failure, this element is not define' % element)

    if not isinstance(el, dict):
        raise Exception(
            'Locating the element:%s is Failure, this element is not define' % element)

    wait = WebDriverWait(g.driver, element_wait_timeout)

    if el['by'].lower() in ('title', 'url', 'current_url'):
        return None
    else:
        try:
            el_location = wait.until(EC.presence_of_element_located(
                (getattr(By, el['by'].upper()), value)))
        except:
            sleep(5)
            try:
                el_location = wait.until(EC.presence_of_element_located(
                    (getattr(By, el['by'].upper()), value)))
            except :
                raise Exception('Locating the element:%s is Failure: Timeout' % element)
    try:
        if g.driver.name in ('chrome', 'safari'):
            g.driver.execute_script(
                "arguments[0].scrollIntoViewIfNeeded(true)", el_location)
        else:
            g.driver.execute_script(
                "arguments[0].scrollIntoView(false)", el_location)
    except:
        pass

    try:
        if action == 'CLICK':
            #增加class_name elements 通过 index定位 点击
            if  isinstance(int(el['remark']),int):
                el_location = {'elements': el['by'], 'value': value, 'index_': int(el['remark'])}  # 反回所需参数到mobile.py
            else:
                el_location = wait.until(EC.element_to_be_clickable(
                    (getattr(By, el['by'].upper()), value)))
        else:
            # elements 输入 remark 是 int 执行index取值
            if isinstance(int(el['remark']), int):
            # if 'class_name' == el['by'] and isinstance(int(el['remark']), int):
                el_location = {'elements': el['by'],'value': value,'index_': int(el['remark'])}  #反回所需参数到mobile.py
            else:
                el_location = wait.until(EC.visibility_of_element_located(
                    (getattr(By, el['by'].upper()), value)))
    except:
        pass

    return el_location


def locating_elements(elements):
    elements_location = {}
    for el in elements:
        elements_location[el] = locating_element(el)
    return elements_location


def locating_data(keys):
    data_location = {}
    for key in keys:
        data_location[key] = locating_element(key)
    return data_location


def locating_air_element(step):
    element = step['element']
    plan_name = g.plan_name
    try:
        el, value = e.get(element)
    except:
        logger.exception(
            'Locating the element:%s is Failure, this element is not define' % element)
        raise Exception(
            'Locating the element:%s is Failure, this element is not define' % element)

    if step['keyword'] in [ 'TAP', 'CHECK', 'NOTCHECK',  'WAIT_']:
        value_list = value.replace(' ', '').replace('，', ',').replace('\n','').split(',')

        # png
        png = [pho for pho in value_list if pho.endswith('.png')]
        photo = png[0] if png else value
        assert photo != '', '当前元素： %s  ,对应图片： %s 未获取到，请检查页面下包含该元素' % (element, photo)
        assert photo.endswith('.png'), '当前元素： %s  ,对应图片： %s 格式不正确，正确 png 格式：photoname.png' % (element, photo)
        photo_path = os.path.join('./data', plan_name, photo)
        assert os.path.exists(photo_path), '当前元素： %s  ,对应图片路径： %s 不存在，请检查该路径下包含该名称的图片' % (element, photo_path)

        # threshold
        thr = [thr for thr in value_list if 'threshold' in thr.lower()]
        threshold_value = thr[0].split('=')[1] if thr else 0.7

        # target_pos
        target_p = [target_p for target_p in value_list if 'target_pos' in target_p.lower()]
        target_pos_value = target_p[0].split('=')[1] if target_p else 5

        # rgb
        rg = [rg for rg in value_list if 'rgb' in rg.lower()]
        rgb_ = rg[0].split('=')[1] if rg else True
        rgb_value = False if not isinstance(rgb_, bool) and 'false' in rgb_.lower() else True

        el_location = photo_path, float(threshold_value), int(target_pos_value), rgb_value

    elif step['keyword'] in ['INPUT']:
        assert type(el) == dict, '当前元素： %s  ,对应value： %s 未获取到，请检查页面下包含该元素' % (element, value)
        el_location = el['by'], value

    elif step['keyword'] == 'SWIPE':
        # 固定坐标滑动
        # assert re.match(r"^(?:[0-9]{1,4}\,){3}[0-9]{1,4}$", value), '当前坐标：%s 格式或数量不对，正确格式如：3000,20,3000,400'% value
        # 相对坐标滑动
        num_list = value.replace('，', ',').split(',')
        # 校验长度
        assert len(num_list)  == 4 , '当前相对坐标：%s 数量不正确 或 未获取到坐标值，正确格式如：0.2131, 0.2, 2, 6' %num_list
        # 校验数值
        error_num_msg = '当前相对坐标：%s 格式不正确，正确格式如：0.2131, 0.2, 2, 6\n' \
                        'w,w,h,h\n' \
                        'width  宽  w1 w2 小数  0.0001 - 19.9999\n' \
                        'height 高  h1 h2 正数  0.0001 - 19.9999' % (value)
        for num in num_list:
            assert type(float(num)) == float and float(num) >= 0.00001 and float(
                num) <= 20, error_num_msg + '\n错误的值：%s' % num

        el_location = num_list
    elif step['keyword'] == 'TAP_POINT':
        # 相对坐标点击
        num_list = value.replace(' ', '').replace('，', ',').split(',')
        poi = [poi for poi in num_list if 'point' in poi.lower()]
        assert len(poi) == 2, '当前相对坐标：%s 数量不正确 或 未获取到坐标值，正确格式如：0.2131, 0.2' %num_list
        point1 = poi[0].split('=')[1]
        point2 = poi[1].split('=')[1]
        tim = [tap_n for tap_n in num_list if 'times' in tap_n.lower()]
        times = tim[0].split('=')[1] if tim else 1

        el_location = [float(point1), float(point2)], int(times)

    elif step['keyword'] == 'SWIPE_PHOTO':
        values = []
        value_ = value.replace('，',',').split(',')
        error_png_msg = '当前图片：%s 格式不正确，正确 图片滑动 格式：photoname.png#photoname1.png'% value
        if len(value_) != 2:
            raise error_png_msg
        for v in value_:
            assert v.endswith('.png'), error_png_msg
            photo_path = os.path.join('./data', plan_name, v)
            assert os.path.exists(photo_path), '当前元素： %s  ,对应图片路径： %s 不存在，请检查该路径下包含该名称的图片' % (element, photo_path)
            values.append(photo_path)
        el_location = values

    elif step['keyword'] in  ['CLICK', 'CHECK_TEXT', 'NOTCHECK_TEXT']:
        assert type(el) == dict, '当前元素： %s  ,对应value： %s 未获取到，请检查页面下包含该元素' % (element, value)
        el_location = el['by'], value

    elif step['keyword'] in ('BACK', 'HOME', 'MENU', 'POWER'):
        el_location = step['keyword']

    else:
        el_location = value

    return el_location


# coding=utf-8

# @Time: 2020/3/2 15:28
# @Auther: liyubin

import shutil

"""
执行端环境清理
"""


def tear_down(git_case=False, snapshot=False):
    file_list = ['data', 'details', 'element', 'JUnit', 'importFiles', 'lib', 'log', 'report', 'testcase']
    if git_case:
        file_list.append('git_case')
    if snapshot:
        file_list.append('snapshot')
    for i in file_list:
        shutil.rmtree(i, ignore_errors=True)



# coding=utf-8

# @Time: 2020/3/11 11:05
# @Auther: liyubin

import re
import datetime
from airtest.core.api import *
from airtest.core.api import swipe as swipe_air # 为统一config关键字，与swipe函数冲突
from super_sweetest.globals import g
from super_sweetest.log import logger
from super_sweetest.locator import locating_air_element
from super_sweetest.config import element_wait_timeout
from super_sweetest.config import rgb_flag
from super_sweetest.servers.ai_server import recognition_ui
from super_sweetest.servers.ai_server import get_words
from super_sweetest.servers.ai_server import is_in_copywriting
from super_sweetest.servers.ai_server import replace_punctuate
from super_sweetest.servers.ai_server import recognition_text
from super_sweetest.servers.ai_server import get_ui_results
from super_sweetest.snapshot import get_air_screenshot
from super_sweetest.servers.common import write_data, read_data


from pathlib import Path
path = Path('lib')
if path.is_dir():
    from lib import *
else:
    from super_sweetest.lib import *


def get_width_height():
    width, height = g.poco.get_screen_size()
    return width, height


def swipe(step):
    """
    1、通过传入的固定坐标滑动

    2、获取设备分辨率计算合适的坐标
    :param poco: 对象
    :param num_list: (0.2, 0.2, 2, 6)
    :return:
    width 宽 w1 w2 小数   0.1-0.9
    height 高  h1 h2 正数 1-9
    """
    num_list = locating_air_element(step)
    duration = step['data'].get('持续时间', 1)


    # 固定坐标滑动
    # start_x = int(num_list[0])
    # start_y = int(num_list[1])
    #
    # end_x = int(num_list[2])
    # end_y = int(num_list[3])
    # if duration:
    #     swipe_air((start_x, start_y), (end_x, end_y), float(duration))
    # else:
    #     swipe_air((start_x, start_y), (end_x, end_y))

    # 相对坐标滑动
    w1 = float(num_list[0])
    w2 = float(num_list[1])
    h1 = float(num_list[2])
    h2 = float(num_list[3])
    width, height = get_width_height()
    start_pt = (float(width * w1), float(height // h1))
    end_pt = (float(width * w2), float(height // h2))

    if duration:
        swipe_air(start_pt, end_pt, float(duration))
    else:
        swipe_air(start_pt, end_pt)
    sleep(1)


def swipe_photo(step):
    value = locating_air_element(step)
    v1 = value[0]
    v2 = value[1]
    swipe_air(Template(filename=v1, rgb=rgb_flag), Template(filename=v2, rgb=rgb_flag))


def tap(step):
    duration = step['data'].get('持续时间', 0.01)
    photo_path, threshold_value, target_pos_value, rgb_value = locating_air_element(step)
    if duration:
        touch(Template(filename=photo_path, threshold=threshold_value, target_pos=target_pos_value, rgb=rgb_value), duration=float(duration))
    else:
        touch(Template(filename=photo_path, threshold=threshold_value, target_pos=target_pos_value, rgb=rgb_value))


def tap_point(step):
    """
    相对坐标点击 TAP_POINT
    point :相对坐标点
    times :点击次数
    :param step:
    :return:
    """
    point, times = locating_air_element(step)
    point1 = point[0]
    point2 = point[1]
    width, height = get_width_height()
    touch([point1*width, point2*height], times=times)


def input(step):
    by, value = locating_air_element(step)
    data = step['data']
    for key in data:
        if by.lower() == 'text':
            if g.platform.lower() == 'air_android':
                g.poco(text=value).set_text(data[key])
            else:
                # ios poco(value).set_text(value)
                g.poco(value).set_text(data[key])
        elif by.lower() == 'textmatches':
            g.poco(textMatches=value).set_text(data[key])
        else:
            if key.startswith('text'):
                text(data['text'], False)
            else:
                text(data[key], False)


def click(step):
    by, value = locating_air_element(step)
    if by.lower() == 'text':
        try:
            if g.platform.lower() == 'air_android':
                g.poco(text=value).click()
            else:
                # ios
                g.poco(value).click()
        except Exception as e:
            sleep(1)
            if g.platform.lower() == 'air_android':
                g.poco(text=value).click()
            else:
                # ios
                g.poco(value).click()
            logger.info('retry poco click, element: %s error msg: %s' % (value, e))
    elif by.lower() == 'textmatches':
        g.poco(textMatches=value).click()
    else:
        assert False, '请指定 element 中 value值：%s 对应的 by 类型为： text'% value


def check(step):
    photo_path, threshold_value, target_pos_value, rgb_value = locating_air_element(step)
    assert exists(Template(filename=photo_path, threshold=threshold_value, target_pos=target_pos_value, rgb=rgb_value))


def notcheck(step):
    photo_path, threshold_value, target_pos_value, rgb_value = locating_air_element(step)
    assert_not_exists(Template(filename=photo_path, threshold=threshold_value, target_pos=target_pos_value, rgb=rgb_value))


def check_text(step):
    by, value = locating_air_element(step)
    if by.lower() == 'text':
        if g.platform.lower() == 'air_android':
            assert g.poco(text=value).exists(), '检查文本 %s 失败'%value
        else:
            # ios
            assert g.poco(value).exists(), '检查文本 %s 失败' % value
    elif by.lower() == 'textmatches':
        assert g.poco(textMatches=value).exists(), '模糊检查文本 %s 失败' % value


def notcheck_text(step):
    by, value = locating_air_element(step)
    if by.lower() == 'text':
        if g.platform.lower() == 'air_android':
            if g.poco(text=value).exists(): raise '反向检查文本 %s 失败'%value
        else:
            # ios
            if g.poco(value).exists(): raise '反向检查文本 %s 失败' % value
    elif by.lower() == 'textmatches':
        if g.poco(textMatches=value).exists(): raise  '模糊反向检查文本 %s 失败' % value


def wait_(step):
    photo_path, threshold_value, target_pos_value, rgb_value = locating_air_element(step)
    wait(Template(filename=photo_path, threshold=threshold_value, target_pos=target_pos_value, rgb=rgb_value), timeout=element_wait_timeout)


def back(step):
    keycode = locating_air_element(step)
    keyevent(keycode)


def home(step):
    keycode = locating_air_element(step)
    keyevent(keycode)


def menu(step):
    keycode = locating_air_element(step)
    keyevent(keycode)


def power(step):
    keycode = locating_air_element(step)
    keyevent(keycode)


def delete():
    # 删除输入框内容
    for i in range(30):
        keyevent('KEYCODE_DEL')


def wake_(step):
    """唤醒手机"""
    wake()


############################AI识别#############################

def write_error_step_text(file_name, time_ms, language, error_word):
    """
    识别错误的文本写到对应步骤的文件 snapshot/planename/AI_zh_error/step_file_time.txt
    :param file:
    :return:
    """
    if len(error_word) > 0:
        text_error_path = os.path.join('snapshot', g.plan_name, 'AI_CopyWrite_' + language + '_Error')
        text_error_path if os.path.exists(text_error_path) else os.mkdir(text_error_path)
        step_file = os.path.join(text_error_path, file_name + '_' + time_ms + '_error_.txt')
        logger.info('验证错误的文本信息已保存在路径： {}'.format(step_file))
        try:
            # 防止频繁写入导致，windows文件系统报错  No such file or directory
            write_data(step_file, str(error_word), mode='w+')
        except Exception as e:
            logger.warning(e)


def update_copyword(file_name, new_words, step_copywrite_num):
    """
    读取旧的文本-去重-写入新的文本
    :param file_name: 文件名称.json
    :param new_words: 文本内容 list []
    :param step_copywrite_num: 对应步骤的当前学习次数
    :return:
    """
    read_data_ = read_data(file_name=file_name, flag='eval')
    learn_num_ = re.findall(r'学习次(.*?).txt', file_name)[0]
    learn_file = file_name.replace(learn_num_, '数_{}_'.format(step_copywrite_num+1)) # 本地文件学习次数+1
    write_data(file_name=learn_file, data_=str(list(set(new_words + read_data_))), mode='w+')
    logger.info('本地对应文案,学习更新后路径： {}'.format(learn_file))
    os.remove(file_name)


def valid_lds_ui(language, file, image_path, step_file, time_ms):
    """
     UI识别
    :param file:
    :param image_path:
    :param step_file:
    :param time_ms:
    :return:
    """
    response_lds_ui = recognition_ui(file)
    get_lds_ui_response = get_ui_results(response_lds_ui)  # 最后先判断UI异常，再判断文本异常
    logger.info('UI识别结果： {} , 提示信息： 当 error 或 score 低于80分时此图很可能显示异常'.format(response_lds_ui))

    # 通过识别后correct / error / default 目录分类保存
    max_name = get_lds_ui_response.get('result')  # 最大值对应的name
    max_score = get_lds_ui_response.get('score')  # 最大值
    max_name_file = os.path.join(image_path, max_name + '_' + language)
    max_name_file if os.path.exists(max_name_file) else os.mkdir(max_name_file)
    lds_ui_name_file = os.path.join(max_name_file, step_file + time_ms + '_' + language + '_' + '.png')  # 按识别分数分类保存
    try:
        import shutil
        shutil.copy(file, lds_ui_name_file)
    except Exception as e:
        logger.warning(e)
    # 提示信息：出现在日志和报告异常中
    screen_file_for_maxname = 'UI识别后图片以最大分数: {0} 分类保存在：{1}'.format(max_name, lds_ui_name_file)
    logger.info(screen_file_for_maxname)
    return max_name, max_score, screen_file_for_maxname


def valid_copywrite(language, file, step_file, step_score, step):
    """
    通过学习次数更新或检查文本内容
    :param file:
    :param step_file:
    :param step_score:
    :return:
    """


    words = recognition_text(file, language)
    words = get_words(words)
    logger.info('识别后文本： {}'.format(words))
    new_words = replace_punctuate(words, baidu_aip=True)  # 忽略百度AIP识别后异常符号

    score_split = step_score.split('学习=')
    learn_num_n = int(score_split[1]) if '文本学习=' in step_score and len(score_split) > 1 else 1  # excel 中 步骤结果：学习次数

    # 精确到步骤的文案 路径 按语言类型分类
    copywrite_file = os.path.join('snapshot', g.plan_name, 'AI_CopyWrite' + language) # 不能按时间保存，否在不能对比
    copywrite_file if os.path.exists(copywrite_file) else os.mkdir(copywrite_file)

    # 遍历本地可能存在的文件名， 文件中最大次数不大于 learn_num_n 值
    now_len_num = '1'  # 本地记录的学习次数
    step_language_path = step_file + '_' + language + '_识别文本记录_学习次数_'
    for learn_num in range(learn_num_n):
        # 先看是否有最大学习次数
        if os.path.exists(os.path.join(copywrite_file, step_language_path + str(learn_num_n) + '_.txt')):
            now_len_num = str(learn_num_n)
            logger.info('文本学习次数已达到： {}'.format(learn_num_n))
            break
        # 小于最大学习次数的文件，从大到小排除
        elif os.path.exists(os.path.join(copywrite_file, step_language_path + str(learn_num_n - learn_num+1) + '_.txt')):
            now_len_num = str(learn_num_n - learn_num+1)
            logger.info('继续进行文本学习： {}'.format(learn_num_n - learn_num+1))
            break
        # 都不符合默认 学习次数1 的路径
        else:
            now_len_num = str(1)
    step_copywrite_file = os.path.join(copywrite_file, step_language_path + now_len_num + '_.txt') # 本地当前存在文件
    # 定义学习次数逻辑
    step_copywrite_num = int(re.findall(r'次数_(.*?)_.txt', step_copywrite_file)[0])  # 文件名中的学习次数

    flag_list = []
    result_list = []
    error_words = []  # 文案识别后错误文本，最后写入错误识别记录
    return_copywrite_data = []
    # 1、当step_copywrite 不存在，按照learn_num_1=1的数值写入
    if not os.path.exists(step_copywrite_file):
        logger.info('本地不存在对应步骤的文案，正在写入...')
        logger.info('文案保存路径：{}'.format(step_copywrite_file))
        write_data(file_name=step_copywrite_file, data_=str(new_words), mode='w+')
    # 2、当文件中数值step_copywrite_num < 定义的学习次数learn_num_n, 本地存在 step_copywrite ，读写后更新内容
    elif step_copywrite_num < learn_num_n and os.path.exists(step_copywrite_file):
        logger.info('对应步骤的文本学习次数还未达到：开始更新...')
        update_copyword(step_copywrite_file, new_words, step_copywrite_num)
    # 3、如果文件中数值step_copywrite_num = 定义的学习次数learn_num_n， 而且本地存在 step_copywrite ， 则满足文本检查的条件， 不再继续写入
    elif step_copywrite_num == learn_num_n:
        logger.info('以首次保存在本地的对应步骤的文案内容 作为 预期结果，检查本次识别是否在 预期结果 中...')
        # 通过文案语言类型对文本识别
        copywrite_data = read_data(step_copywrite_file, flag='eval', mode='r')
        logger.info('文案内容的路径： {}'.format(step_copywrite_file))
        logger.info('文案内容： {}'.format(copywrite_data))

        # 忽略的部分文本
        filter_str = step.get('expected', '').get('#filter', '@#$%')
        filter_str_list = filter_str.split('&') # 包含多个&分隔的过滤词语时
        for word in new_words:
            filter_in_word = [True for filter_s in filter_str_list if filter_s in word] # 防止双重循环，判断忽略词是否在word中
            filter_flag = True if len(filter_in_word) == 0 else False # 如果列表长度=0说明，不忽略
            if filter_flag: # 如果忽略字符不在当前识别word，开始检查
                # 识别后词语的文案检查
                flag = is_in_copywriting(word, copywrite_data)
                result = '识别结果： {0},  文本： {1} '.format(flag, word)
            else:  # 忽略检查filter_str的word文本
                flag = True
                result = '忽略文本： {0}， 识别的文本： {1}'.format(filter_str, word)
            logger.info(result)
            flag_list.append(flag)
            result_list.append(result)
            error_words.append(word) if not flag else ''

        return_copywrite_data = copywrite_data
    return flag_list, result_list, error_words, new_words, return_copywrite_data  # 逻辑判断后记得最后必须返回否则接收不到参数报错


def ai_(step, testcase):
    """
    AI智能识别
    :param step:
    :param testcase:
    :return:
    """
    # 自动化步骤中是否需要跳过
    step_score = step.get('score', 'no step_score') # 包含 #AI识别 / 文本学习=3
    if '#AI识别' in step_score:
        logger.info('########## 已跳过AI识别... ##########\n')
        return True
    logger.info('--------------------------- AI识别开始... ---------------------------\n')

    # 通过图像中文本识别出语言,   用'AUTO_DETECT': '自动检测语言 识别中英文效果好
    language_type = {
        'CHN_ENG': '中英文混合', 'ENG': '英文', 'JAP': '日语', 'KOR': '韩语', 'FRE': '法语', 'SPA': '西班牙语', 'POR': '葡萄牙语',
        'GER': '德语', 'ITA': '意大利语', 'RUS': '俄语', 'DAN': '丹麦语', 'DUT': '荷兰语', 'MAL': '马来语', 'SWE': '瑞典语', 'IND': '印尼语', 'POL': '波兰语',
        'ROM': '罗马尼亚语', 'TUR': '土耳其语', 'GRE': '希腊语', 'HUN': '匈牙利语'
    }
    PLAN_NAME_LANGUAGE = '文本识别需要在 testcase、element、data 的文件名称中配置语言类型，如：test01登录用例#CHN_ENG-TextCase.xlsx \n支持的语言类型：{}'.format(language_type)

    assert '#' in g.plan_name, PLAN_NAME_LANGUAGE
    language = g.plan_name.split('#')[1]  # 默认未设置时，自动识别语言，但是会大大降低识别准确性

    assert language.upper() in language_type.keys(), PLAN_NAME_LANGUAGE
    logger.info('设定的语言: {0} - {1}'.format(language, language_type.get(language.upper())))


    # 路径生成
    plan_name = g.plan_name                              # 项目名称
    sheet_name = g.sheet_name                            # sheet名称
    case_id = testcase.get('id', 'no case_id')           # 用例编号
    case_title = testcase.get('title', 'no case_title')  # 用例标题
    step_num = re.findall(r'^\d*', step.get('no', 'no step_num').replace('>','').replace('^','').replace('<',''))[0]  # 当前用例标题的步骤数,正则匹配数字

    # 精确到步骤的图片路径
    step_file = plan_name + '_' + sheet_name + '_' + case_id + '_' + case_title + '_' + step_num + '_'
    today = datetime.datetime.now().strftime('%Y%m%d_')
    image_path_day = os.path.join('snapshot', g.plan_name, 'AI_UI_' + today + '_' + language) # 每天
    image_path_day if os.path.exists(image_path_day) else os.mkdir(image_path_day)
    time_ms = datetime.datetime.now().strftime('%Y%m%d_%H%M_%S%f')  # 含微秒的日期时间
    file = os.path.join(image_path_day, step_file + time_ms + '_' + language + '.png')
    logger.info('截图保存方式： 项目名称_sheet名称_用例编号_用例标题_步骤数字_日期时间.png')
    logger.info('截图所在位置： {}'.format(file))

    # 图片截取
    get_air_screenshot(file)

    # 检查图片是否存在 隐式等待
    while True:
        time.sleep(0.01)
        if (datetime.datetime.now() - datetime.datetime.now()).seconds > 10:
            break
        if os.path.exists(file):

            if '#UI识别' in step_score:
                logger.info('\n########## 已跳过UI识别... ##########\n')
                max_name, max_score, screen_file_for_maxname = '已跳过UI识别...', 0, file
            else:
                logger.info('--------------------------- UI识别... ---------------------------\n')
                max_name, max_score, screen_file_for_maxname = valid_lds_ui(language, file, image_path_day, step_file, time_ms)

            if '#文本识别' in step_score:
                logger.info('\n########## 已跳过文本识别... ##########\n')
                flag_list, result_list, error_words, new_words, return_copywrite_data = [], [], '', '', ''
            else:
                logger.info('--------------------------- 文本识别... ---------------------------\n')
                flag_list, result_list, error_words, new_words, return_copywrite_data = valid_copywrite(language, file, step_file, step_score, step)  # 文件夹不能以每日保存，不然无法比对

                # 文案识别错误的文本单独记录
                write_error_step_text(step_file, time_ms, language, error_words)

                output_text = {'text': new_words if new_words else '识别到的内容为空'}
                if output_text:
                    logger.info('output_text: %s' % repr(output_text))
                    my_func = step.get('output', '').get('func', None)   # 获取输出数据中自定义函数
                    if my_func:
                        logger.info('开始执行自定义函数... 识别后的文本信息在自定义函数中通过key值text提取')
                        eval(str('{0}({1})'.format(my_func.replace('(', '').replace(')', ''), output_text)), globals(), locals()) # 输出数据中调用自定义函数,去除（）

            # 图片识别结束抛出异常，报告中展示
            result_list.append(screen_file_for_maxname)

            # 检查UI识别 与 文本识别结果，抛出异常
            ui_flag = max_name == 'correct' and float(max_score) > 80  if max_name != '已跳过UI识别...' else True# 最大值name是correct是True, 分数阈值 80
            text_flag = False not in flag_list
            assert (ui_flag and text_flag), ('\nUI识别结果：{0}  \n文本识别结果：{1}  \n本次识别到的文本：{2}  \n本地文案预期内容：{3}'.format(str({'result': max_name, 'score': max_score}), result_list, new_words, return_copywrite_data))  # 文本识别结果检查
            break





# coding=utf-8

# @Time: 2020/3/5 10:57
# @Auther: liyubin

from PIL import Image
import math
import operator
from functools import reduce
import base64

"""
check photo
"""


def resize_photo(filename):
    """
    resize photo
    :param filename:
    :return:
    """
    img = Image.open(filename)
    width = img.size[0]
    height = img.size[1]
    region = img.resize((int(width * 0.5), int(height * 0.5)), Image.ANTIALIAS)
    region.save(filename)


def base64_photo(filename, resize=False):
    """
    photo base64
    :param filename:
    :return:
    """
    if resize:
        resize_photo(filename)
    with open(filename, 'rb')as fp:
        base64_data = 'data:image/png;base64,' + base64.b64encode(fp.read()).decode()
    return base64_data


def compare(pic1, pic2):
    """
    :param pic1: file1
    :param pic2: file2
    :return: differ
    """
    image1 = Image.open(pic1)
    image2 = Image.open(pic2)

    histogram1 = image1.histogram()
    histogram2 = image2.histogram()

    differ = math.sqrt(
        reduce(operator.add, list(map(lambda a, b: (a - b) ** 2, histogram1, histogram2))) / len(histogram1))

    print(differ)
    return differ


if __name__ == '__main__':

    differ = compare(r'./1.png', r'./2.png')
    if differ == 0.0:
        print('ok: ', differ)
    elif differ < 3.16:
        print('小于3.16: ', differ)
    else:
        print('fail: ', differ)





# coding: utf-8

# @Time:2020/3/15 12:34
# @Auther:sahala


from setuptools import setup, find_packages  # 这个包没有的可以pip一下

setup(
    name="super_sweetest",  # 这里是pip项目发布的名称
    version="1.2.3.2",  # 版本号，数值大的会优先被pip
    keywords=("pip", "super_sweetest", "featureextraction"),
    description="automation tools",
    long_description="增加运行sweet.plan(row_num=[37, 38])指定行用例，让用例调试加速，试用API/MQTT/ANDROID/IOS/WEB等",
    license="MIT Licence",

    url="http://github.com/SahalaProject/super_sweetest",  # 项目相关文件地址，一般是github
    author="liyubin",
    author_email="1399393088@qq.com",
    ########################
    # 精确控制打包文件
    # packages=find_packages("super_sweetest"),
    # package_dir = {"": "super_sweetest"},
    # package_data = {
    #     "": ["*.py", "example/*.zip"], # 根目录下的文件
    #     # "example": ["*.zip"]
    # },
    # exclude_package_data={"": ["README.md"]}, # 过滤打包文件
    ########################
    packages=find_packages(), # 默认加载py
    include_package_data=True,

    platforms="any",
    install_requires=[
                      "Appium-Python-Client",
                      "certifi",
                      "chardet",
                      "idna",
                      "injson",
                      "Pillow",
                      "requests",
                      "selenium",
                      "urllib3",
                      "xlrd",
                      "XlsxWriter",
                      "airtest",
                      "pocoui",
					  "validator_sa",
					  "paho-mqtt",
					  "openpyxl",
					  "arrow",
					  "requests_toolbelt",
					  "GitPython",
					  "redis",
					  "baidu-aip",
                      ],  # 这个项目需要的第三方库
    entry_points={
            'console_scripts': [
                'super_sweetest = client.cli:main'
            ]
        }, # 命令行直接启动的命令
)



# 步骤：
# 使用说明：https://setuptools.readthedocs.io/en/latest/setuptools.html#including-data-files

# 1.setup.py放在被打包同级
    # 本地打包项目文件
    # python setup.py sdist

# 2.上传项目到pypi服务器
    # pip install twine
    # twine upload dist/name.tar.gz

# 3.下载上传的库
    # pip install name


# 4.安装打包的本地文件
    # tar -xzvf super.tar.gz
    # python setup.py install


# coding=utf-8

# @Time: 2020/7/23 15:14
# @Auther: liyubin

import os
import unittest
from super_sweetest import Autotest
from super_sweetest.servers.common import write_data, read_data

"""
独立执行模块
"""

class TESTCASE(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        case_data = read_data('./run_case_data.json', 'json', 'r+')
        cls.test_name = case_data.get('test_name', '')

    def test_run(self):
        """
        测试主方法
        项目名称，和测试用例、页面元素表文件名称中的项目名称必须一致
        """
        global msg
        plan_name = self.test_name

        # 环境配置信息
        if os.path.exists('testcase/' + plan_name + '-TestCase.xlsx'):
            # 测试用例集文件里的Sheet表单名称
            sheet_name = '*'
            desired_caps_ = {'platformName': 'mqtt'}
            desired_caps = desired_caps_
            server_url = ''

            msg = '数据准备正常'
        else:
            desired_caps = {}
            server_url = ''
            sheet_name = ''
            msg = '未获取有效数据'

        # 初始化自动化实例
        sweet = Autotest(plan_name, sheet_name, desired_caps, server_url)
        # 执行自动化测试
        sweet.plan()
        # 保存txt格式的详情信息
        write_data('details/details.txt', sweet.report_data, 'json', 'w+')

    @classmethod
    def tearDownClass(cls):
        # 写入执行完成状态，数据处理模块确认状态后上传报告
        write_data('./run_case_data.json', {'status': True, 'msg': msg}, flag='json', mode='w+')


if __name__ == '__main__':
    unittest.main()



# coding:utf-8

# @Time: 2020/3/1 13:33
# @Auther: liyubin

import os
import sys
import time
import json
import datetime
import shutil
import configparser
import requests
import redis
from git import Repo
from super_sweetest.servers.teardown import tear_down
from super_sweetest.servers.common import read_data, write_data

"""
数据处理模块
"""

config_file = './config.ini'
# 创建配置文件对象
config = configparser.RawConfigParser()
# 读取配置文件内容
config.read(config_file, encoding='utf-8')
# ATP服务端的URL地址
PLATFORM_URL = "".join(map(chr, json.loads(dict(config.items('ATP-Server'))['platform_url'])))
# 服务端Redis的IP地址
REDIS_IP = "".join(map(chr, json.loads(dict(config.items('ATP-Server'))['redis_ip'])))
REDIS_KEY = "".join(map(chr, json.loads(dict(config.items('ATP-Server'))['redis_key'])))
REDIS_PORT = "".join(map(chr, json.loads(dict(config.items('ATP-Server'))['redis_port'])))
# 存放测试用例的git仓库
GIT_URL = "".join(map(chr, json.loads(dict(config.items('ATP-Server'))['git_url'])))
if GIT_URL.count('@') - 1 != 0:
    GIT_URL = GIT_URL.replace('@', '%40', GIT_URL.count('@') - 1)

# app执行配置
EXECUTION_ID = dict(config.items('ATP-Server'))['execution_id']
# 设备名称
DEVICE_NAME = dict(config.items('ATP-Server'))['name'].split('_')[1].replace('##', ':')
# iostagen_file
IOSTANGE_FILE = "".join(map(chr, json.loads(dict(config.items('ATP-Server'))['iostagen_file'])))

TEST_TYPE = dict(config.items('ATP-Server'))['test_type']


class CmdColors:
    HEADER = '\033[95m'
    BLUE = '\033[94m'
    GREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    END = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'


# 连接 Redis 数据库
POOL = redis.ConnectionPool(host=REDIS_IP, port=REDIS_PORT, db=REDIS_KEY, decode_responses=True)
REDIS_POOL = redis.Redis(connection_pool=POOL)


def write_log_none():
    """日志内容清空，预防日志上传到平台重复"""
    today_log = os.path.join(os.getcwd(), 'log', time.strftime('%Y%m%d.log'))
    new_log = os.path.join(os.getcwd(), 'log', 'new.log')
    write_data(today_log, '')
    write_data(new_log, '')


def mkdir_and_del(srcpath):
    if os.path.isdir(srcpath):
        shutil.rmtree(srcpath)
    os.mkdir(srcpath)


def reset_file(file_name, test_project, test_environment):
    case_path = os.path.join(os.getcwd(), "git_case")
    device_lower = DEVICE_NAME.lower()
    if 'iphone' in device_lower or 'lds' in device_lower or 'ipad' in device_lower:
        lib_dst_path = os.path.join(case_path, test_project + "-Project-ios", test_environment, file_name)
    else:
        lib_dst_path = os.path.join(case_path, test_project + "-Project-android", test_environment, file_name)
    if file_name != 'snapshot/expected/':  # 不删除预期图片，避免git拉取压力大，手动放置图片
        # 确定Git用例存储库中是否有一个lib_dst_path目录
        if os.path.isdir(lib_dst_path):
            lib_src_path = os.path.join(os.getcwd(), file_name)
            # 确定执行目录下是否有一个lib_src_path目录
            if os.path.isdir(lib_src_path):
                shutil.rmtree(lib_src_path)
            shutil.copytree(lib_dst_path, lib_src_path)


def all_mkdir_copy(test_project, test_environment):
    reset_file(file_name="lib", test_project=test_project, test_environment=test_environment)
    reset_file(file_name="data", test_project=test_project, test_environment=test_environment)
    reset_file(file_name="element", test_project=test_project, test_environment=test_environment)
    reset_file(file_name="testcase", test_project=test_project, test_environment=test_environment)
    reset_file(file_name="files", test_project=test_project, test_environment=test_environment)
    mkdir_and_del(srcpath=os.path.join(os.getcwd(), "JUnit"))
    mkdir_and_del(srcpath=os.path.join(os.getcwd(), "report"))
    mkdir_and_del(srcpath=os.path.join(os.getcwd(), "details"))


def read_log_file():
    """
    从以日期为维度统计的日志中提取出本次的日志文件，保存至log/new11111111111.log
    """
    # 初始日志文件的起始行号
    # initial_log_line = 0
    # 初始日志文件的日期前缀
    # initial_log_time = ''
    # 如果日志文件的日期前缀不是当天的日期
    # if initial_log_time != time.strftime('%Y%m%d'):
    # 重置日志文件的前缀、起始行数
    # initial_log_time = time.strftime('%Y%m%d')

    initial_log_line = 0
    # 读取日志文件的全部字符串
    with open(os.path.join(os.getcwd(), 'log', time.strftime('%Y%m%d.log')), "r", encoding='utf-8') as f:
        # 将日志字符串按换行符拆成列表
        log_file_data = f.read().split('\n')
    # 如果起始日志行反而比最大行数都大的话
    if initial_log_line > len(log_file_data):
        # 重置日志文件的起始行数
        initial_log_line = 0
    # 按行号增量取出本次测试的日志
    log_file_list = log_file_data[initial_log_line:]
    # 更新日志文件的起始行号
    initial_log_line = len(log_file_data)
    # 存储日志信息的字典
    log_file_dict = {}
    # 用这个变量控制每一行都是不重复的
    previous_row = ''
    # 整理每一行日志
    for i in range(len(log_file_list)):
        # 不读取空行
        if log_file_list[i] == '':
            continue
        # 不读取和上一行重复的数据
        if log_file_list[i] == previous_row:
            continue
        # 判断当前行是否包含特殊分割符
        if ']: #  ' in log_file_list[i]:
            # 将日志信息按用途分类
            log_file_dict[i] = {
                'time': log_file_list[i][:23],
                'grade': log_file_list[i][25:log_file_list[i].find(']')],
                'message': log_file_list[i][log_file_list[i].find(']') + 6:],
            }
        else:
            # 如果是错误信息，上传特殊格式
            log_file_dict[i] = {
                'time': '',
                'grade': '',
                'message': log_file_list[i].replace(' ', '&nbsp;'),
            }
        # 更新上一行的数据
        previous_row = log_file_list[i]

    # 在log目录下写入新的日志文件
    with open(os.path.join('log', 'new.log'), "w", encoding='utf-8') as fp:
        json.dump(log_file_dict, fp)


def post_heartbeat(status, localtime=None, task_dict=None, remark=None):
    """
    # 描述:
        发送心跳请求到ATP服务器（失败重试，直至恢复连接）
    # 用法0（第一次）:
        post_heartbeat(status=0)
    # 用法1（正常）:
        post_heartbeat(status=1, localtime=[调用时间])
    # 用法2（任务无法完成/放弃任务，需要在备注中提交错误信息）:
        post_heartbeat(status=2, localtime=[调用时间], task_dict=[任务字典], remark=[备注])
    # 用法3（开始执行任务）:
        post_heartbeat(status=3, localtime=[调用时间], task_dict=[任务字典])
    # 用法4（任务执行完成，上传测试报告）:
        post_heartbeat(status=4, localtime=[调用时间], task_dict=[任务字典])
    # 用法5（执行端异常，需要在备注中提交异常信息）:
        post_heartbeat(status=5, remark=[备注])
    """

    # 已连接手机list 设备离线设置平台离线 6
    device_list = read_data('../device_list.json', 'json', 'r')
    if DEVICE_NAME not in device_list:
        status = 6

    if not localtime:
        localtime = time.asctime(time.localtime(time.time()))
    # ATP平台端上执行端心跳的API接口
    heartbeat_api_url = PLATFORM_URL + 'software/execution/heartbeat/'
    # 心跳请求连接次数
    connect = 0
    # 开始发送心跳请求
    while True:
        # 捕获异常
        try:
            # 执行端心跳参数
            up_data = {'id': EXECUTION_ID, 'status': status, 'task_dict': json.dumps(task_dict),
                       'remark': remark}
            # 状态为4时，组织待上传的次数报告文件
            if status == 4:
                # 拔高变量的层级
                files_report = None
                files_junit = None
                # 取出XXX-Report@XXXXXX_XXX.xlsx的测试报告文件名
                for _root, _dirs, files in os.walk(os.path.join('report', task_dict['test_name'])):
                    if files:
                        files_report = files[0]
                # 取出XXX-Report@XXXXXX_XXX.xml的测试汇总文件名
                for _root, _dirs, files in os.walk('JUnit'):
                    if files:
                        files_junit = files[0]
                # 读取测试报告文件
                files_report_obj = read_data(os.path.join('report', task_dict['test_name'], files_report), '', 'rb')
                # 读取测试汇总文件
                files_junit_obj = read_data(os.path.join('JUnit', files_junit), '', 'rb')
                # 读取测试详情文件
                files_details_obj = read_data(os.path.join('details', 'details.txt'), '', 'rb')
                # 读取测试日志文件
                read_log_file()
                with open(os.path.join('log', 'new.log'), 'rb') as fp4:
                    files_log_obj = fp4.read()
                # 汇总XXX-Report@XXXXXX_XXX.xlsx、XXX-Report@XXXXXX_XXX.xml、details.txt文件对象
                files_obj = {
                    'file_obj': (str(task_dict['queue_id']) + '.xlsx', files_report_obj),
                    'file_obj_junit': (str(task_dict['queue_id']) + '.xml', files_junit_obj),
                    'file_obj_details': (str(task_dict['queue_id']) + '.txt', files_details_obj),
                    'file_obj_log': (str(task_dict['queue_id']) + '.log', files_log_obj)
                }
            else:
                files_obj = {}
            # 向ATP平台端发送执行端心跳请求
            requests_data = requests.post(heartbeat_api_url, data=up_data, files=files_obj)
            requests_json = requests_data.json()
            # 判断平台端心跳状态处理结果
            if requests_json['code'] == 200:
                # 心跳请求响应成功，结束死循环
                print('[Heartbeat][' + localtime + ']> ' + str(requests_json['data']))
                break
            # 服务器内部错误错误
            else:
                print('[Heartbeat Error][' + localtime + ']> > ' + str(requests_json))
                break
        # 网络请求错误
        except Exception as exc:
            print('[Heartbeat Error][' + localtime + ']> ' + str(exc))
        # 30秒后重新发送心跳请求
        time.sleep(30)
        connect += 1
        print('[Heartbeat Error][' + localtime + ']> 进行第 ' + str(connect) + ' 次重新连接')
        # 防止 异常后文件找不到 进入死循环 join argument must be str or bytes, not 'NoneType'
        if connect > 2:
            break


def write_testdata_and_run(test_name, connect_type):
    """
     写入测试所需数据
    :param test_name:
    """

    connect_type_dict = json.loads(connect_type)

    if TEST_TYPE == '1':
        # 如果平台没同步当前设备connect_type_dict / {"iphone":"ios"} 时，手动输入
        connect_type_dict[DEVICE_NAME] = [input('当前手机未在平台同步connect_type，请输入：')] if connect_type_dict.get(DEVICE_NAME) == "" or 'iphone' in connect_type_dict else connect_type_dict.get(DEVICE_NAME)
        # android
        run_case_data = {'test_name': test_name, 'status': False, 'device_name': DEVICE_NAME,
                         'connect_type_list': connect_type_dict.get(DEVICE_NAME)}
        write_data('./run_case_data.json', run_case_data, 'json', 'w+')

        # 启动test_case模块
        os.popen('start python ./test_case.py')
    else:
        # ios
        run_case_data = {'test_name': test_name, 'status': False, 'device_name': DEVICE_NAME,
                         'iostagen_file':IOSTANGE_FILE}
        write_data('./run_case_data.json', run_case_data, 'json', 'w+')

        # 启动test_case模块
        os.system('nohup python3 ./test_case.py &')

    # 如果运行结束状态status：True，继续下一步
    # 隐式等待/避免执行端假死
    now_time_flag = datetime.datetime.now()
    while True:
        time.sleep(5)
        if (datetime.datetime.now() - now_time_flag).seconds > 3600:
            break
        else:
            status = read_data('./run_case_data.json', 'json', 'r')
            if status['status']:
                return status.get('msg', '')


def run_test_main(execution_id, localtime):
    """
    执行app主方法
    :param queu_type:
    :param test_type:
    """
    print(localtime + CmdColors.HEADER + " →_→ 发现有新任务" + CmdColors.END)
    value_dict = json.loads(REDIS_POOL.lpop(execution_id))
    # 上报开始执行任务的心跳
    post_heartbeat(status=3, localtime=localtime, task_dict=value_dict)

    try:
        print('[Queue Task Dict]> ' + str(value_dict))

        print(CmdColors.BLUE + "重置 Sweetest 测试运行环境 ……" + CmdColors.END)
        all_mkdir_copy(test_project=value_dict['test_project'], test_environment=value_dict['test_environment'])

        print(CmdColors.GREEN + "Sweetest 测试运行环境就绪, 开始执行 ……%s " % CmdColors.END)
        global run_msg
        run_msg = write_testdata_and_run(test_name=value_dict['test_name'], connect_type=value_dict.get('connect_type','{"iphone":"ios"}'))

        print('[App Server Run Info]> X-Sweetest 测试完成并上传测试报告')
        # 上报任务执行完成的心跳
        post_heartbeat(status=4, localtime=localtime, task_dict=value_dict)
        print('[App Server Run Info]> 测试报告上传完毕')

        # 清空日志
        write_log_none()
    except Exception as exc:
        # 上报任务无法完成的心跳
        post_heartbeat(status=2, localtime=localtime, task_dict=value_dict, remark=str(exc)+ '---' +str(run_msg))
        print('[App Server Run Error]> ' + str(exc))


def pull_case():
    """
    拉取最新case
    每次执行用例时pull，保持数据同步
    """
    try:
        git_case_path = os.path.join(os.getcwd(), "git_case")
        if os.path.isdir(git_case_path):
            print(CmdColors.BLUE + " *_* 发现 git_case 目录, 执行更新最新用例流程 ……" + CmdColors.END)
            repo = Repo.init(path=git_case_path)
            remote = repo.remote()
            remote.pull()
            print(CmdColors.GREEN + " ^_^ 已更新 GitLab 仓库中 git_case 项目的用例" + CmdColors.END)
        else:
            print(CmdColors.FAIL + " @_@ 找不到 git_case 目录, 执行下载新用例流程 ……" + CmdColors.END)
            Repo.clone_from(url=GIT_URL, to_path=git_case_path)
            print(CmdColors.GREEN + " ^_^ 已下载 GitLab 仓库中 git_case 项目的用例" + CmdColors.END)
    except Exception as e:
        print(' --- 用例下载失败，请检查Git-Server配置信息 --- \n\t%s' % e)
    time.sleep(3)


def main_():
    """
    主运行程序
    :return:
    """
    # mkdir_and_del(srcpath=os.path.join(os.getcwd(), "log"))
    # mkdir_and_del(srcpath=os.path.join(os.getcwd(), "snapshot"))  #不删除历史日志和预期图片
    tear_down(git_case=False, snapshot=False)  # 如需提交代码，取消注释执行

    logo = """
   :f###.    ;#:
     ,###    #:
      G##,  G,             X-ATP执行终端
      .### :#              APP自动化测试
        # DW
        :##:               监听Redis数据库:
         # .               {0}
        G:###
       tE ,##,             atp服务运行地址:
      ,#   G##:            {1}
     .#:    ###
    :#;     :###.
    """.format(REDIS_IP, PLATFORM_URL)
    print(CmdColors.BLUE + logo + CmdColors.END)
    pull_case()

    print('[App Server Run Info]> 检测是否有历史任务未完成')
    # 上报第一次/刚启动执行端的心跳
    post_heartbeat(status=0)

    while True:
        time.sleep(5)
        try:
            length = REDIS_POOL.llen(EXECUTION_ID)
            localtime = time.asctime(time.localtime(time.time()))
            if length > 0:
                pull_case()
                run_test_main(EXECUTION_ID, localtime)
            else:
                # 上报正常的心跳
                post_heartbeat(status=1, localtime=localtime)
            # 调用Flush，从内存中刷入日志文件
            sys.stdout.flush()
        except Exception as exc:
            # 上报执行端异常的心跳
            post_heartbeat(status=5, remark=str(exc))
            print('[App Server Run Error]> ' + str(exc))



# coding:utf-8

import os
import sys
import json
import time
import base64
import shutil
import configparser
from pathlib import Path
from super_sweetest.client.mqtt.mqtt_package.run_api_mqtt import MqttRun


def mqtt_workspace(sys_name, data, redis_host=None):
    """
    创建mqtt自动化测试执行工作空间并自动/手动执行
    """
    data_dict = {
        'name': data['name'],
        'platform_url': data['platform_url'],
        'git_url': data['git_url'],
        'redis_ip': data['redis_ip'] if not redis_host else redis_host,
        'redis_key': data['redis_key'],
        'redis_port': data['redis_port'],
        'execution_id': data['execution_id'],
    }
    current_path = os.path.abspath(__file__)
    current_dir = Path(os.path.dirname(current_path))
    cwd_dir = Path.cwd()
    work_space = cwd_dir / ('Api_Mqtt_Work_%s' % data_dict['name'])
    os.mkdir(str(work_space))
    create_profile(data=data_dict, work_space=work_space, current_dir=current_dir)
    print('\n###   工作空间   ###\nApi_Mqtt_Work_%s\n' % data_dict['name'])
    # 后台执行atp执行端
    if sys_name == 'Linux':
        daemon_start(data=data_dict, work_space=work_space)
        print('###   X-ATP自动化测试执行端服务已在后台运行   ###')
    else:
        print('###   您需要手动启动执行端服务   ###\ncd Api_Mqtt_Work_%s\npython run_this.py\n'% data_dict['name'])


def create_profile(data, work_space, current_dir):
    """
    在程序执行的当前目录下创建配置文件和执行程序
    """
    # 处理GIT仓库URL里的特殊字符
    data['git_url'] = data['git_url'].replace('%40', '@')
    config = configparser.ConfigParser()
    # 创建config配置文件
    file = 'config.ini'
    config.read(work_space / file)
    config.add_section('ATP-Server')
    # 平台URL、GIT仓库URL、工作空间名、队列IP、队列KEY、队列端口号、注册ID
    config.set('ATP-Server', 'platform_url', json.dumps(list(map(ord, data['platform_url']))))
    config.set('ATP-Server', 'name', data['name'])
    config.set('ATP-Server', 'git_url', json.dumps(list(map(ord, data['git_url']))))
    config.set('ATP-Server', 'redis_ip', json.dumps(list(map(ord, data['redis_ip']))))
    config.set('ATP-Server', 'redis_key', json.dumps(list(map(ord, str(data['redis_key'])))))
    config.set('ATP-Server', 'redis_port', json.dumps(list(map(ord, str(data['redis_port'])))))
    config.set('ATP-Server', 'execution_id', str(data['execution_id']))
    # 保存config配置文件
    with open(work_space / file, 'w', encoding='utf-8') as configfile:
        config.write(configfile)
    # 拷贝 run_this  test_case 执行文件到工作空间 , 因为引用的 MqttRun 来自 super_sweetest 所以下面拷贝文件必须在 super_sweetest 下
    shutil.copy(current_dir / 'copy_file' / 'run_this.py', work_space / 'run_this.py')
    shutil.copy(current_dir / 'copy_file' / 'test_case.py', work_space / 'test_case.py')


def daemon_start(data, work_space, stdin='/dev/null', stdout='/dev/null', stderr='/var/log/x-client-mqtt-server.error'):
    """
    启动守护进程
    """
    # fork出一个子进程，父进程退出
    try:
        pid = os.fork()
        # 父进程退出函数
        if pid > 0:
            return
    except OSError as e:
        sys.stderr.write("[mqtt Server Error]> 第一次 fork 失败, " + e.strerror)
        os._exit(1)
    # 创建日志存放目录，设置日志输出文件
    log_file = Path("atp_log")
    if not log_file.is_dir():
        os.mkdir(str(log_file))
    sys.stdin = open(stdin, 'r')
    sys.stdout = open(stdout, 'a+')
    sys.stderr = open(stderr, 'a+')
    # 等待父进程结束
    time.sleep(2)
    # 父进程退出后，子进程由init收养
    # setsid使子进程成为新的会话首进程和进程组的组长，与原来的进程组、控制终端和登录会话脱离
    os.setsid()
    # 防止在类似于临时挂载的文件系统下运行，例如/mnt文件夹下，这样守护进程一旦运行，临时挂载的文件系统就无法卸载了，这里我们推荐把当前工作目录切换到根目录下
    os.chdir(work_space)
    # 设置用户创建文件的默认权限，设置的是权限“补码”，这里将文件权限掩码设为0，使得用户创建的文件具有最大的权限。否则，默认权限是从父进程继承得来的
    os.umask(0)
    # 第二次进行fork,为了防止会话首进程意外获得控制终端
    try:
        pid = os.fork()
        if pid > 0:
            # 父进程退出
            os._exit(0)
    except OSError as e:
        sys.stderr.write("[mqtt Server Error]> 第二次 fork 失败, " + e.strerror)
        os._exit(1)
    # 此时改程序已经是守护进程了，再执行需要后台执行的程序即可
    r = MqttRun(data)
    r.run()



# coding=utf-8

# @Time: 2020/4/10 16:42
# @Auther: liyubin


from airtest.core.api import *
from poco.drivers.ios import iosPoco
from super_sweetest.servers.ios_server import kill_ios_tagent_server
from super_sweetest.servers.ios_server import run_ios_tagent_server
from super_sweetest.servers.ios_server import run_iproxy8100
from super_sweetest.servers.ios_server import post_iproxy_status

########################################
# from airtest.core.settings import Settings

# 全局伐值

# Settings.THRESHOLD_STRICT = 0.6  # assert_exists语句的默认阈值，一般比THRESHOLD更高一些
# Settings.THRESHOLD = 0.6  # 其他语句的伐值
# Settings.OPDELAY = 1  # 每一步操作后等待1秒再进行下一步操作
# Settings.FIND_TIMEOUT = 10  # 图像查找超时时间
# Settings.CVSTRATEGY = ["surf", "tpl", "brisk"]  # 修改图片识别算法顺序，只要成功匹配任意一个符合设定阙值的结果，程序就会认为识别成功

########################################


"""
airtest ios  服务
"""


class AirIos():

    def setup_air_ios(self, device_name, app_package, desired_caps):
        """
        connect_device airtest， init poco
        """

        auto_setup(__file__)

        # 获取配置
        iostagen_file = desired_caps.get('iostagen_file', '未配置')
        platform = desired_caps.get('platform', '未配置')

        # 服务启动 获取iproxy status
        iproxy_status = self.setup_server(iostagen_file, platform, device_name)

        # 二次启动，失败后抛出异常
        if iproxy_status != 'success':
            sec_iproxy_status = self.setup_server(iostagen_file, platform, device_name)
            assert sec_iproxy_status == 'success', '当前手机 %s 启动服务失败, 请检查手机连接状态，重启手机重装' \
                                                   '和手机上WebDriverAgent应用已信任，' \
                                                   '请确认当前手机已在此Mac电脑启动WebDriverAgent成功，' \
                                                   '并成功连接airtest' % device_name

        iproxy_url = 'http://127.0.0.1:8100'  # 端口转发 iproxy 8100 8100

        # 或者使用connect_device函数
        connect_device("ios:///" + iproxy_url)

        # poco服务,先连接设备再初始化
        poco = iosPoco()

        start_app(app_package)

        sleep(8)
        return poco

    @staticmethod
    def setup_server(iostagen_file, platform, devicename):
        """airtest ios前置条件server"""
        kill_ios_tagent_server(devicename)
        run_iproxy8100()
        run_ios_tagent_server(iostagen_file, platform, devicename)
        iproxy_status = post_iproxy_status()
        return iproxy_status

    @staticmethod
    def teardown_air_ios(g):
        """关闭app，停止poco实例"""
        stop_app(g.appPackage)
        kill_ios_tagent_server(g.deviceName)

if __name__ == '__main__':
    desired_caps = {'iostagen_file': '/Users/liyubin1/Desktop/iOS-Tagent11',
                    'platform': 'iOS',
                    }

    deviceName = 'LDS-Z-C5197'
    app_package = 'com.apple.Preferences'

    air = AirIos()
    air.setup_air_ios(deviceName, app_package, desired_caps)



# coding:utf-8

# @Time: 2020/7/7 17:00
# @Auther: liyubin

import paho.mqtt.client as mqtt
from super_sweetest.globals import g
from super_sweetest.log import logger
from super_sweetest.keywords.mqtt import write_mqtt_msg

"""
mqtt client 非阻塞客户端 / 初始化MQTT客户端 / 设置全局变量 self.mqtt_client_
"""


class MqttClient:

    def __init__(self, host, port, mqttClient):
        self._host = host
        self._port = port
        self._mqttClient = mqttClient

    def on_mqtt_connect(self, username, password):
        """非阻塞连接MQTT服务器"""
        self._mqttClient.tls_set()  # 配置网络加密和身份验证选项。启用S​​SL / TLS支持
        self._mqttClient.username_pw_set(username, password)
        self._mqttClient.connect_async(self._host, self._port, 60)
        self._mqttClient.loop_start()

    def on_subscribe(self, topic, qos):
        """订阅和消息回调"""
        self._mqttClient.subscribe(topic, qos)

    def on_unsubscribe(self, topic, qos):
        """ 取消订阅客户一个或多个主题"""
        self._mqttClient.unsubscribe(topic, qos)

    def on_publish(self, topic, msg, qos):
        """publish 发布消息"""
        self._mqttClient.publish(topic, msg, qos)

    def on_message_come(self, client, userdata, msg):
        """从服务器收到发布消息时的回调"""
        recv_msg = msg.payload.decode('utf-8')
        logger.info('Recv ---- 回调 topic 主题: ' + msg.topic + ' --------- 收到消息: ' + str(recv_msg))
        # 写入返回值（由于异常在mqtt回调函数中不会被抛出）
        write_mqtt_msg(recv_msg)

    def on_message(self):
        """消息回调"""
        self._mqttClient.on_message = self.on_message_come  # 调用 on_message_come 实际调用on_message

    def on_disconnect(self, client, userdata, rc):
        """断开"""
        if rc != 0:
            logger.info('Unexpected disconnection %s' % rc)

    def discionnect(self):
        """断开主方法"""
        self.on_disconnect(self._mqttClient, userdata=None, rc='')  # 调用 on_message_come 实际调用on_message


####################初始化MQTT客户端####################

class Mqtt_Init:

    def __init__(self, _data):
        self.data = _data
        self.mqtt_host_ = self.data.get('mqtt_host', 'no')
        self.mqtt_host = self.mqtt_host_.split(':')[0] if ":" in self.mqtt_host_ else 'no'
        self.mqtt_port = self.mqtt_host_.split(':')[1] if ":" in self.mqtt_host_ else 'no'
        self.client_Id = self.data.get('client_Id', 'no')
        self.transport = self.data.get('transport', 'no')
        self.mqtt_username = self.data.get('mqtt_username', 'no')
        self.mqtt_password = self.data.get('mqtt_password', 'no')

    def setup_mqttclient(self):
        """
        初始化 类 MqttClient 客户端连接实例/通过实例调用MqttClient类方法
        """
        # 参数key校验
        demo = {'mqtt_host': 'test.com:8443', 'client_Id': 'value', 'transport': 'websockets/tcp',
                'mqtt_username': 'value', 'mqtt_password': 'value'}
        assert 'no' not in (self.mqtt_host_, self.mqtt_host, self.mqtt_port, self.client_Id, self.transport,
                            self.mqtt_username, self.mqtt_password), '\nmqtt 登录参数格式有误 \n当前参数：%s \n正确格式: %s' % (str(self.data), str(demo))
        # 登录
        mqttClient = mqtt.Client(client_id=self.client_Id, clean_session=True, protocol=4, transport=self.transport)
        client = MqttClient(self.mqtt_host, int(self.mqtt_port), mqttClient)
        client.on_mqtt_connect(self.mqtt_username, self.mqtt_password)
        g.mqtt_client_ = client # 设置全局变量 self.mqtt_client_
        logger.info("--- Mqtt Connect Success ---")

    @staticmethod
    def teardown_mqttclient(client):
        """断开mqtt客户端连接"""
        client.discionnect()

#######################################################
